package controller;

import java.awt.Point;
import java.util.ArrayList;
import model.WorldModel;
import model.WorldObject;
import view.ModelView;

/**
 * Controller of the model/view. The controller should be used to interact with 
 *  the model and view
 * @author sjoerdlagarde
 */
public class WorldController {
    
    protected WorldModel model;
    protected ModelView view;
    
    /**
     * Creates a new controller with the given model and view
     * @param model The model to use
     * @param view The view to use
     */
    public WorldController(WorldModel model, ModelView view) {
        this.model = model;
        this.view = view;
    }
    
    /**
     * Sets a status message
     * @param status The status message
     */
    public void setStatusMessage(String status) {
        if ( view != null )
            view.setStatusMessage(status);
    }
    
    /**
     * Moves an agent around
     * @param agent The agent to move
     * @param movement The movement to make
     */
    public void moveAgent(int agent, AgentMovement.Move movement) {
        int[] move = AgentMovement.getAgentMovement(movement);
        Point currentLocation = model.getAgentLocation(agent);
        if ( currentLocation != null ) 
            model.setAgentLocation(agent, currentLocation.x+move[0], currentLocation.y+move[1]);
    }
    
    /**
     * Returns all neighbour cells of the agent
     * @param agent The agent
     * @return A list of all neighbour cells of the agent. The cells are listed top, right, 
     *  bottom and left, respectively. If a cell is not available (e.g. agent is in a corner), 
     *  a null value is used
     */
    public ArrayList<WorldObject> getNeighbourCells(int agent) {
        Point currentLocation = model.getAgentLocation(agent);
        ArrayList<WorldObject> neighbours = new ArrayList<WorldObject>();
        if ( currentLocation != null ) {
            neighbours.add( model.getLocation(currentLocation.x, currentLocation.y-1) );      // Top
            neighbours.add( model.getLocation(currentLocation.x+1, currentLocation.y) );      // Right
            neighbours.add( model.getLocation(currentLocation.x, currentLocation.y+1) );      // Bottom
            neighbours.add( model.getLocation(currentLocation.x-1, currentLocation.y) );      // Left
        } 
        
        return neighbours;
    }
    
    /**
     * Returns the world object the agent is currently standing on
     * @param agent The agent
     * @return The world object at the agent's location or null if the agent is invalid
     */
    public WorldObject getCurrentCell(int agent) {
        Point currentLocation = model.getAgentLocation(agent);
        if ( currentLocation != null )
            return model.getLocation(currentLocation.x, currentLocation.y);
        return null;
    }
    
    /**
     * Returns the world object at a given location
     * @param x The x-value of the location
     * @param y The y-value of the location
     * @return The object at the given location, or null when the location is invalid
     */
    public WorldObject getWorldObjectAt(int x, int y) {
        return model.getLocation(x, y);
    }
    
    /**
     * Adds a world object to a given location
     * @param object The object to add
     * @param x The x-value of the location
     * @param y The y-value of the location
     */
    public void addWorldObject(WorldObject object, int x, int y) {
        model.addWorldObject(object, x, y);
    }
    
    /**
     * Removes a world object from a given location
     * @param x The x-value of the location
     * @param y The y-value of the location
     */
    public void removeWorldObject(int x, int y) {
        model.removeWorldObject(x, y);
    }
    
}
