package model;

import java.awt.Point;
import java.util.Observable;
import view.ModelMessagesQueue;

/**
 *
 * @author sjoerdlagarde
 */
public class WorldModel extends Observable {

    private WorldObject[][] world;
    private Point[] agentPositions;
    private AgentObject[] agents;
    
    private ModelMessagesQueue msgs;
    
    /**
     * Creates a new, empty model with a given width and height and a given number of agents
     * @param width The width of the model
     * @param height The height of the model
     * @param numberOfAgents The number of agents
     */
    public WorldModel(int width, int height, int numberOfAgents) {
        width = ( width<0 ) ? 1 : width;
        height = ( height<0 ) ? 1 : height;
        numberOfAgents = ( numberOfAgents<0 ) ? 1 : numberOfAgents;
        world = new WorldObject[height][width];
        agentPositions = new Point[numberOfAgents];
        
        agents = new AgentObject[numberOfAgents];
        for ( int i=0; i<numberOfAgents; i++ ) {
            agents[i] = new AgentObject();
            agents[i].setCurrentObject(new EmptyObject());
        }
        
        msgs = ModelMessagesQueue.getInstance();
        msgs.addMessage("Constructed new model with width "+width+" and height "+height);
        
        resetWorld();
    }
    
    /**
     * Resets the world to an empty world
     */
    private void resetWorld() {
        for ( int i=0; i<world.length; i++ ) {
            for ( int j=0; j<world[i].length; j++ )
                world[i][j] = new EmptyObject();
        }
        
        // Put agents in position 0,0
        for ( int i=agentPositions.length-1; i>=0; i-- ) {
            agentPositions[i] = new Point(0,0);
            if ( world[0][0] != null )
                agents[i].setCurrentObject(world[0][0]);
            world[0][0] = agents[i];
        }
        
        msgs.addMessage("Reset world");
        this.setChanged();
        this.notifyObservers();
    }
    
    /**
     * Returns the dimensions of the world
     * @return An 2D-array of integers, representing the (idx: 0) y and (idx: 1) x dimensions of the world
     */
    public int[] getWorldDimensions() {
        return new int[]{world.length,world[0].length};
    }
    
    /**
     * Returns the number of agents that are currently placed in the world
     * @return The number of agents in the world
     */
    public int getNumberOfAgents() {
        return agentPositions.length;
    }
    
    /**
     * Sets the location of a specific agent
     * @param agent The identifier of the agent to set the location of
     * @param location The location to set for the agent
     */
    public void setAgentLocation(int agent, int x, int y) {
        msgs.addMessage("Trying to move agent "+agent+" to location ("+x+", "+y+")...");
        if ( !isOpenLocation(agent, x, y) ) {
            msgs.addMessage("Move agent "+agent+" failed: location ("+x+","+y+") is not open");
            return;
        }
        if ( !isValidAgent(agent) ) {
            msgs.addMessage("Move agent "+agent+" failed: agent "+agent+" is not a valid agent");
            return;
        }

        Point oldPosition = new Point(agentPositions[agent].x, agentPositions[agent].y);        
        world[oldPosition.y][oldPosition.x] = agents[agent].getCurrentObject();     // Put back the object the agent is standing on
        agentPositions[agent].x = x;
        agentPositions[agent].y = y;
        agents[agent].setCurrentObject(world[y][x]);
        world[y][x] = agents[agent];
        
        msgs.addMessage("Agent "+agent+" moved from ("+oldPosition.x+", "+oldPosition.y+") to ("+agentPositions[agent].x+", "+agentPositions[agent].y+")");
        this.setChanged();
        this.notifyObservers();
    }
    
    /**
     * Returns the location of an agent
     * @param agent The agent to retrieve the location from
     * @return The location of the agent as a Point, or null when the agent is invalid
     */
    public Point getAgentLocation(int agent) {
        if ( !isValidAgent(agent) )
            return null;
        return agentPositions[agent];
    }
    
    /**
     * Returns whether or not a location is valid (i.e. not out of world boundaries)
     * @param x The x-value of the location to check
     * @param y The y-value of the location to check
     * @return True when the location is valid, false otherwise
     */
    private boolean isValidLocation(int x, int y) {
        return x>=0 && x<world[0].length && y>=0 && y<world.length;
    }
    
    /**
     * Returns whether or not an agent is valid (i.e. valid identifier)
     * @param agent The agent to check
     * @return True when the agent is valid, false otherwise
     */
    private boolean isValidAgent(int agent) {
        return ( agent>=0 && agent<agentPositions.length );
    }

    /**
     * Returns whether or not a location is open to visit (e.g. it could be blocked by obstacles)
     * @param agent The index of the agent that is checking whether or not a location is open
     * @param x The x-value of the location to check
     * @param y The y-value of the location to check
     * @return True when the location is open to visit, false otherwise
     */
    public boolean isOpenLocation(int agent, int x, int y) {
        if ( !isValidLocation(x, y) ) 
            return false;
        if ( agent>=0 && agents[agent].blocksLocation() ) {
            for ( int i=0; i<agentPositions.length; i++ ) 
                if ( agent!=i && agentPositions[i].x == x && agentPositions[i].y == y )
                    return false;
        }

        return ( !world[y][x].blocksLocation() );
    }

    /**
     * Returns whether or not a location is open to visit (e.g. it could be blocked by obstacles)
     * @param x The x-value of the location to check
     * @param y The y-value of the location to check
     * @return True when the location is open to visit, false otherwise
     */    
    public boolean isOpenLocation(int x, int y) {
        return isOpenLocation(-1, x, y);
    }
    
    /**
     * Returns the WorldObject from a specific location
     * @param x The x-value of the location to return
     * @param y The y-value of the location to return
     * @return The WorldObject at the given location, or null when the location is invalid
     */
    public WorldObject getLocation(int x, int y) {
        if ( !isValidLocation(x, y) ) 
            return null;
        return world[y][x];
    }
    
    /**
     * Adds a WorldObject to a specific location
     * @param object The WorldObject to add to the location
     * @param x The x-value of the location to add the object to
     * @param y The y-value of the location to add the object to
     */
    public void addWorldObject(WorldObject object, int x, int y) {
        msgs.addMessage("Adding world object '"+object.getName()+"' at location ("+x+", "+y+")");
        if ( !isOpenLocation(x, y) ) {
            msgs.addMessage("Adding object failed: location ("+x+", "+y+") is already occupied");
            return;
        } 
        object.setPreviousObject(world[y][x]);
        world[y][x] = object;
        
        this.setChanged();
        this.notifyObservers();
    }
    
    /**
     * Removes a WorldObject from a specific location
     * @param x The x-value of the location to remove the object from
     * @param y The y-value of the location to remove the object from
     */
    public void removeWorldObject(int x, int y) {
        msgs.addMessage("Removing world object from location ("+x+", "+y+")");
        if ( !isValidLocation(x, y) ) 
            return;
        
        WorldObject previousObject = world[y][x].getPreviousObject();
        world[y][x] = ( previousObject == null ) ? new EmptyObject() : previousObject;
        
        this.setChanged();
        this.notifyObservers();
    }
    
}
