package view;

import java.awt.Color;
import java.awt.Graphics;
import java.awt.Image;
import java.io.IOException;
import javax.imageio.ImageIO;
import world.WorldConfiguration;

/**
 *
 * @author sjoerdlagarde
 */
public class WorldObjectDrawer {
    
    Graphics g;
    private int locationX;
    private int locationY;
    private int squareSize;
    
    public WorldObjectDrawer(Graphics g) {
        this.g = g;
        squareSize = WorldConfiguration.gridSquareSize;
    }
    
    /**
     * Sets the location of the cell in which to draw. The location is indicated 
     *  by the top left corner of the cell
     * @param x The x-value of the location
     * @param y The y-value of the location
     */
    public void setLocation(int x, int y) {
        locationX = x;
        locationY = y;
    }
    
    /**
     * Draws an image
     * @param filename The filename of the image to draw
     * @param color The color that will be used to draw a circle when the image cannot be found
     */
    public void drawImage(String filename, Color color) {
        try {
            Image img = ImageIO.read(this.getClass().getClassLoader().getResource(filename));
            //Image img = ImageIO.read(new File(filename));
            int wh = WorldConfiguration.gridSquareSize/2;
            int[] drawPos = calculateDrawPosition();
            g.drawImage(img, drawPos[0], drawPos[1], wh, wh, null);          
        } catch (IOException ex) {
            drawCircle(color);
        }
    }
    
    /**
     * Gives a cell in the grid of the world a background color
     * @param color The color of the background
     */
    public void drawBackground(Color color) {
        Color oldColor = g.getColor();
        g.setColor(color);
        int wh = WorldConfiguration.gridSquareSize;
        g.fillRect(locationX*wh+1, locationY*wh+1, wh-1, wh-1);
        g.setColor(oldColor);
    }
    
    /**
     * Draws a circle with a given color (filled)
     * @param color The color to fill the circle with
     */
    public void drawCircle(Color color) {
        Color oldColor = g.getColor();
        g.setColor(color);
        int wh = WorldConfiguration.gridSquareSize/2;
        int[] drawPos = calculateDrawPosition();
        g.fillOval(drawPos[0], drawPos[1], wh, wh);
        g.setColor(oldColor);
    }
    
    /**
     * Draws text 
     * @param text The text to draw
     * @param color The color of the text
     */
    public void drawText(String text, Color color) {
        Color oldColor = g.getColor();
        g.setColor(color);
        int[] drawPos = calculateDrawPosition();
        g.drawString(text, drawPos[0], drawPos[1]);
        g.setColor(oldColor);
    }
    
    /**
     * Calculates the positions in cells to draw shapes to
     * @return An array of ints containing the x-value (idx: 0) and y-value (idx: 1) of the positions to draw to
     */
    private int[] calculateDrawPosition() {
        return new int[]{locationX*squareSize + squareSize/4, locationY*squareSize + squareSize/4};
    }
}
